class PlayerBehavior extends Sup.Behavior {
	
	private speed: number = 0.02;
	private angle: number = 180 * RAD;
	private isFast: boolean = false;
	private lastProut: number = 0;

	solidBodies: Sup.ArcadePhysics2D.Body[] = [];
	items: Sup.Actor[] = [];

	awake() {
		g_player = this;
		
		// Retrieve the solid elements to collide with the player
		this.solidBodies = Sup.ArcadePhysics2D.getAllBodies();
		
		// Remove the items from the solid elements
		let length = this.solidBodies.length;
		for (let i = 0; i < length; i++) {
			let name = this.solidBodies[i].actor.getName();
			
			//Sup.log("name :", name);
			if (name.indexOf("RoomHider") > -1 || name.indexOf("Patron") > -1 || name.indexOf("Riz") > -1 || name.indexOf("Banane") > -1 || name.indexOf("Café") > -1) {
				this.solidBodies.splice(i, 1);
				i--;
				length--;
			}
		}
		
		// Retrieve the items to intersect with the player
		this.items = Sup.getActor("Items").getChildren();
		
		// Set the sprite of the player depending of its gender
		this.actor.spriteRenderer.setSprite("Sprites/Player/Sprite_" + g_gender);
	}

	update() {
		// Remove the incorrect or destroyed solid bodies
		let length = this.solidBodies.length;
		for (let i = 0; i < length; i++) {
			if (this.solidBodies[i] === undefined || this.solidBodies[i].isDestroyed()) {
				this.solidBodies.splice(i, 1);
				i--;
				length--;
			}
		}
		
		// Enable the collision between the player and all the scene actors
		Sup.ArcadePhysics2D.collides(this.actor.arcadeBody2D, this.solidBodies);
		
		// Get the current velocity of the player
		let velocity = this.actor.arcadeBody2D.getVelocity();
		
		// Cancel if the game is paused
		if (g_pause) {
			velocity.x = 0;
			velocity.y = 0;
			this.actor.arcadeBody2D.setVelocity(velocity);
			this.actor.spriteRenderer.setAnimation("Idle");
			return;
		}
		
		let speed = (this.isFast) ? this.speed * 1.5 : this.speed;
		
		// Change the player velocity on the vertical axis
		if (Sup.Input.isKeyDown("UP") || Sup.Input.isKeyDown("Z") || Sup.Input.isKeyDown("W")) {
			velocity.y = speed;
		} else if (Sup.Input.isKeyDown("DOWN") || Sup.Input.isKeyDown("S")) {
			velocity.y = -speed;
		} else {
			velocity.y = 0;
		}
		
		// Change the player velocity on the horizontal axis
		if (Sup.Input.isKeyDown("LEFT") || Sup.Input.isKeyDown("Q") || Sup.Input.isKeyDown("A")) {
			velocity.x = -speed;
		} else if (Sup.Input.isKeyDown("RIGHT") || Sup.Input.isKeyDown("D")) {
			velocity.x = speed;
		} else {
			velocity.x = 0;
		}
		
		// Reset the player velocity if it can't move
		if (g_blockPlayerMove) {
			velocity.x = 0;
			velocity.y = 0;
		}
		
		// Set the player sprite animation
		if (velocity.x != 0 || velocity.y != 0) {
			this.actor.spriteRenderer.setAnimation("Walk", true);
		} else {
			this.actor.spriteRenderer.setAnimation("Idle");
		}
		
		// Set the player rotation depending on its direction
		if (velocity.y < 0) {
			if (velocity.x < 0) {
				this.angle = 135 * RAD;
			} else if (velocity.x > 0) {
				this.angle = 225 * RAD;
			} else {
				this.angle = 180 * RAD;
			}
		} else if (velocity.y > 0) {
			if (velocity.x < 0) {
				this.angle = 45 * RAD;
			} else if (velocity.x > 0) {
				this.angle = 315 * RAD;
			} else {
				this.angle = 0 * RAD;
			}
		} else {
			if (velocity.x < 0) {
				this.angle = 90 * RAD;
			} else if (velocity.x > 0) {
				this.angle = 270 * RAD;
			}
		}
		
		// Apply an extremly small velocity when the player doesn't move so it collides with moving targets
		if (velocity.x == 0) {
			velocity.x = 0.000001;
		}
		if (velocity.y == 0) {
			velocity.y = 0.000001;
		}
		
		// Apply the player rotation
		this.actor.setEulerZ(this.angle);
		
		// Apply the player velocity
		this.actor.arcadeBody2D.setVelocity(velocity);
		
		// Do a prout if we're beyond the last prout level
		if (g_ui.health <= g_proutLevels[this.lastProut]) {
			this.actor.getChild("Prout").spriteRenderer.setAnimation("Moving").playAnimation(false);
			this.lastProut++;
			
			Sup.Audio.playSound("Sounds/Sound_Fart", g_volume_sfx);
			
			if (g_currentRoom != null && Sup.ArcadePhysics2D.intersects(this.actor.arcadeBody2D, g_currentRoom.arcadeBody2D))  {
				Sup.log("Fart in room !");
			} else {
				g_proutLives--;
				Sup.log("Fart outside room !");
			}
		}
		
		// Check the intersection of the player with items
		for (let item of this.items) {
			if (item.isDestroyed() || item.arcadeBody2D === undefined) {
				continue;
			}
			
			if (Sup.ArcadePhysics2D.intersects(this.actor.arcadeBody2D, item.arcadeBody2D)) {
				//Sup.log("Player hits item :", item.getName());
				
				item.destroy();
				
				// Apply the items effects to the player
				if (item.getName().indexOf("Riz") > -1) {
					g_ui.health += g_maxTime * 0.10;
					Sup.Audio.playSound("Sounds/Sound_Powerup", g_volume_sfx);
					
				} else if (item.getName().indexOf("Banane") > -1) {
					g_ui.health -= g_maxTime * 0.10;
					Sup.Audio.playSound("Sounds/Sound_Malus", g_volume_sfx);
					
				} else if (item.getName().indexOf("Café") > -1) {
					g_ui.health -= g_maxTime * 0.05;
					Sup.Audio.playSound("Sounds/Sound_Café", g_volume_sfx);
					
					this.isFast = true;
					
					// Disable the speedup after 2 seconds
					Sup.setTimeout(2000, () => {
						this.isFast = false;
					});
				}
				
				// Reset the last prout as the health may have gonne higher than a prout level
				for (let i = 0; i < this.lastProut; i++) {
					if (g_ui.health > g_proutLevels[i]) {
						this.lastProut = i;
						break;
					}
				}
			}
		}
	}
}
Sup.registerBehavior(PlayerBehavior);
